@@ -37,11 +37,7 @@ pub const unsafe fn simd_extract<T, U>(x: T, idx: u32) -> U;
 /// `idx` must be in-bounds of the vector.
 #[rustc_nounwind]
 #[rustc_intrinsic]
-pub unsafe fn simd_insert_dyn<T, U>(mut x: T, idx: u32, val: U) -> T {
-    // SAFETY: `idx` must be in-bounds
-    unsafe { (&raw mut x).cast::<U>().add(idx as usize).write(val) }
-    x
-}
+pub const unsafe fn simd_insert_dyn<T, U>(x: T, idx: u32, val: U) -> T;
 
 /// Extracts an element from a vector.
 ///
@@ -54,10 +50,7 @@ pub unsafe fn simd_insert_dyn<T, U>(mut x: T, idx: u32, val: U) -> T {
 /// `idx` must be in-bounds of the vector.
 #[rustc_nounwind]
 #[rustc_intrinsic]
-pub unsafe fn simd_extract_dyn<T, U>(x: T, idx: u32) -> U {
-    // SAFETY: `idx` must be in-bounds
-    unsafe { (&raw const x).cast::<U>().add(idx as usize).read() }
-}
+pub const unsafe fn simd_extract_dyn<T, U>(x: T, idx: u32) -> U;
 
 /// Adds two simd vectors elementwise.
 ///
